function fun = Matern_corr(nu)

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

if ~isinf(nu)
    if ~mod(nu-1/2,1) == 0
        error('Matrn-Korrelationsmodell ist nur fr ganzzahlige Werte von nu-1/2 implementiert')
    end
end

switch nu
    case 0.5 % exponential correlation model
        fun = @(Dw)exp(-Dw);
    case 1.5
        fun = @(Dw)(1+sqrt(3)*Dw).*exp(-sqrt(3)*Dw);
    case 2.5
        fun = @(Dw)(1+sqrt(5)*Dw+5/3*(Dw).^2).*exp(-sqrt(5)*Dw);
    case 3.5
        fun = @(Dw)(1+sqrt(7)*Dw+14/5*Dw.^2+7*sqrt(7)/15*Dw.^3).*exp(-sqrt(7)*Dw);
    case 4.5
        fun = @(Dw)(1+3*Dw+27/7*Dw.^2+18/7*Dw.^3+81/105*Dw.^4).*exp(-3*Dw);
    case 5.5
        fun = @(Dw)(1+sqrt(11)*Dw+44/9*Dw.^2+11*sqrt(11)/9*Dw.^3+121/63*Dw.^4+121*sqrt(11)/945*Dw.^5).*exp(-sqrt(11)*Dw);
    case inf % square-exponential correlation model
        fun = @(Dw)exp(-Dw.^2/2);
    otherwise
        error('Matrn-Korrelationsmodell nur fr nu=0.5, 1.5, 2.5, 3.5, 4.5, 5.5 und ''inf'' implementiert')
end

end