function y=f_MAP_multiD(input)

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

% load parameters
load('MAP_par');

if strcmp(MAP_par.learn,'L_c')
    
    [~,~,idd]   = unique(MAP_par.iso_learn,'stable');
    L_c                   = MAP_par.L_c;
    L_c(MAP_par.id_learn) = input(idd);
    g_mic                 = MAP_par.g_mic;
elseif strcmp(MAP_par.learn,'g_mic')
    L_c   = MAP_par.L_c;
    g_mic = input;
else
    input_L_c = input(1:end-1);
    [~,~,idd]   = unique(MAP_par.iso_learn,'stable');
    L_c                   = MAP_par.L_c;
    L_c(MAP_par.id_learn) = input_L_c(idd);
    g_mic                 = input(end);
end

if min(L_c)<eps
    y = inf;
elseif max(L_c)>1000
    y = inf;
elseif g_mic>1
    y = inf;
elseif g_mic<0
    y = inf;
else
    
    M=MAP_par.M;
    mu0=MAP_par.mu0;
    k0=MAP_par.k0;
    a0=MAP_par.a0;
    b0=MAP_par.b0;
    si_eps=MAP_par.si_eps;
    R_fun_meso = MAP_par.R_fun_meso;
    
    % weighted distance of measurement locations
    D_m = zeros(size(M,1),size(M,1),size(M,2)-1);
    for ii = 1:size(M,2)-1
        D_m(:,:,ii) = pdist2(M(:,ii+1),M(:,ii+1));
    end
    Dwm = sqrt(sum((D_m./permute(L_c,[1 3 2])).^2,3));
    
    % number of measurements
    n_m=size(M,1);
    
    % noise matrix
    R_eps = si_eps^2*eye(n_m);
    
    % correlation matrix of measurement locations
    R_m = g_mic*(Dwm==0)+(1-g_mic)*R_fun_meso(Dwm)+R_eps;
    
    % calculations according to IALCCE paper
    R1_inv  = sum(inv(R_m),'all');
    R1y_inv = ones(1,n_m)*(R_m\M(:,1));
    Ry_inv  = M(:,1)'*(R_m\M(:,1));
    det_Rm   = det(R_m);
    
    kn=k0+R1_inv;
    % mun=(k0*mu0+R1y_inv)/kn;
    an=a0+n_m/2;
    zeta_n=Ry_inv+(k0*mu0^2*R1_inv-2*k0*mu0*R1y_inv-R1y_inv.^2)./kn;
    bn=b0+zeta_n/2;
    
    
    % uniform prior for correlation parameters
    f_logtau = 0;
    % uninformative gamma prior for correlation lengths (a=1/2, b=0)
    %     f_logtau = -1/2*sum(log(L_c(MAP_par.id_learn)));
    % lognormal prior for correlation length (mu=10, si=10)
    %     mu = 10;
    %     si = 10;
    %     muln=log(mu^2./sqrt(si^2+mu^2));
    %     siln=sqrt(log(si^2/mu^2+1));
    %     f_logtau = -sum(log(L_c(MAP_par.id_learn)))-1/2*sum((log(L_c(MAP_par.id_learn))-muln).^2/siln^2);
    
    y=log(kn)+2*an.*log(bn)+log(det_Rm)-2*f_logtau;
    
    if ~isreal(y)
        y=inf;
    end
    
end


