function [mu_pp,lambda_pp_inv,nu_t,NG_pp]=t_postpred(mu0,k0,a0,b0,R_fun,M,x,si_eps,L_c,BU_type)

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------

Berechnung der posterior predictive Verteilung fr rumlich korrelierte Daten einer Normalverteilung
Das Bayes'sche Update wird in zwei Schritten durchgefhrt:
	1: Berechnung der posterior Parameter der Normal-Gamma Verteilung (conjugate prior)
    2: Berechnung der Parameter der posterior predictive Verteilung (Student's t-Verteilung)

%}

% Kurzform
val=M(:,1)';
n=size(val,2);
v1=ones(1,n);

% A-priori Korrelation der Datenpositionen
D_mm = zeros(size(M,1),size(M,1),size(M,2)-1);
for ii = 1:size(M,2)-1
    D_mm(:,:,ii) = pdist2(M(:,ii+1),M(:,ii+1));
end
Dwmm = sqrt(sum((D_mm./permute(L_c,[1 3 2])).^2,3));
R_mm=R_fun(Dwmm)+eye(size(M,1))*si_eps^2;

% Posterior parameter der Normal-Gamma Verteilung
k_n=k0+v1*(R_mm\v1');
mu_n=(k0*mu0+v1*(R_mm\val'))/(k_n);
a_n=a0+n/2;
b_n=b0+1/2*(val*(R_mm\val')+(k0*mu0^2*v1*(R_mm\v1')-2*k0*mu0*v1*(R_mm\val')-(v1*(R_mm\val'))^2)/k_n);

NG_pp.a_n = a_n;
NG_pp.b_n = b_n;
NG_pp.mu_n = mu_n;
NG_pp.k_n = k_n;

% Parameter der posterior predictive Student's t-Verteilung
if strcmp(BU_type,'marg')
    
    mu_pp=zeros(size(x,1),1);
    lambda_pp_inv=zeros(size(x,1),1);
    ratio=0.1;
    
    for ii=1:size(x,1)
        
        % A-priori Korrelation des Diskretisierungspunkte mit den Datenpositionen
        D_xm = zeros(1,size(M,1),size(M,2)-1);
        for jj = 1:size(M,2)-1
            D_xm(1,:,jj) = pdist2(x(ii,jj),M(:,jj+1));
        end
        Dwxm = sqrt(sum((D_xm./permute(L_c,[1 3 2])).^2,3));
        R_xm=R_fun(Dwxm);
        
        % posterior predictive Parameter
        mu_pp(ii)=mu_n+R_xm*(R_mm\(val-mu_n)');
        lambda_pp_inv(ii)=b_n/a_n*(1-R_xm*(R_mm\R_xm')+(1-R_xm*(R_mm\v1'))^2*k_n^(-1));
        
        % Fortschritt
        if ii/size(x,1)>=ratio
            fprintf('\n%g%% des marginalen Updates erfolgreich\n',ratio*100)
            ratio=ratio+0.1;
        end
    end
    
else
    
    % A-priori Korrelation der Diskretisierungspunkte mit den Datenpositionen
    D_xm = zeros(size(x,1),size(M,1),size(M,2)-1);
    for ii = 1:size(M,2)-1
        D_xm(:,:,ii) = pdist2(x(:,ii),M(:,ii+1));
    end
    Dwxm = sqrt(sum((D_xm./permute(L_c,[1 3 2])).^2,3));
    R_xm=R_fun(Dwxm);
    
    % A-priori Korrelation der Diskretisierungspunkte
    D_xx = zeros(size(x,1),size(x,1),size(x,2));
    for ii = 1:size(x,2)
        D_xx(:,:,ii) = pdist2(x(:,ii),x(:,ii));
    end
    Dwxx = sqrt(sum((D_xx./permute(L_c,[1 3 2])).^2,3));
    R_xx=R_fun(Dwxx);
    
    % posterior predictive Parameter
    mu_pp=mu_n+R_xm*(R_mm\(val-mu_n)');
%     lambda_pp = a_n/b_n*inv(R_xx-R_xm*(R_mm\R_xm')+(1-R_xm*(R_mm\v1'))*k_n^(-1)*(1-R_xm*(R_mm\v1'))')^(-1);
    lambda_pp_inv = b_n/a_n*(R_xx-R_xm*(R_mm\R_xm')+(1-R_xm*(R_mm\v1'))*k_n^(-1)*(1-R_xm*(R_mm\v1'))');

end

% Freiheitsgrade der Student's t-Verteilung
nu_t=2*a_n;
end