function [mu_SA,si_SA,Si_SA,z1_m,z2_m]=Parameter_SA_2D(iz1,iz2,zz1_SA,zz2_SA,obj_RF,Si_SA,n_Gp)

%{
--------------------------------------------------------------------------
Created by:                         Date:           Comment:
Sebastian Geyer                     Juli 2021         ---
Technische Universitaet Muenchen
s.geyer@tum.de
--------------------------------------------------------------------------
%}

L_z1 = obj_RF.L(1);
L_z2 = obj_RF.L(2);

% number of averaging elements per dimension
n_z = size(zz1_SA')-1;

% element id
id_el = (iz1-1)*n_z(2)+iz2;

% size of averaging element
l_z1 = L_z1/n_z(1);
l_z2 = L_z2/n_z(2);

% averaging domain
z1_0 = (iz1-1)*l_z1;
z1_1 = iz1*l_z1;
z2_0 = (iz2-1)*l_z2;
z2_1 = iz2*l_z2;
z1_m = (z1_0+z1_1)/2;
z2_m = (z2_0+z2_1)/2;

% Gauss points and weights for the numerical integration
[xi_Gp,w_Gp] = Gauss_quad(n_Gp); % Gauss-Legendre points and weights on standardized grid

z1_Gp = (xi_Gp+1)*l_z1/2+z1_0; % Gauss-Legendre points in z1
z2_Gp = (xi_Gp+1)*l_z2/2+z2_0; % Gauss-Legendre points in z2

% 2D grid of quadrature points and corresponding weights
[zz1,zz2] = meshgrid(z1_Gp',z2_Gp);
w_Gp_2D = w_Gp.*w_Gp';

% quadrature weights for quadruple integration
w_Gp_4D = w_Gp_2D(:)*w_Gp_2D(:)';

if obj_RF.RF_hom
    
    % mean at quadrature nodes
    mu_Gp = obj_RF.mu_n*ones(size(zz1(:)));
    
    % correlation of quadrature nodes
    D_Gp = zeros(size(zz1(:),1),size(zz1(:),1),2);
    D_Gp(:,:,1) = pdist2(zz1(:),zz1(:));
    D_Gp(:,:,2) = pdist2(zz2(:),zz2(:));
    DwGp = sqrt(sum((D_Gp./permute(obj_RF.L_c,[1 3 2])).^2,3));
    R_Gp=obj_RF.R_fun(DwGp);
    
    % covariance matrix of quadrature nodes
    Si_Gp = obj_RF.si_n^2*R_Gp;
    
else % RF update on averaging element
    [mu_Gp,lambda_t_inv,nu_t,~]=t_postpred(obj_RF.mu0,obj_RF.k0,obj_RF.a0,obj_RF.b0,obj_RF.R_fun,obj_RF.data_update,[zz1(:) zz2(:)],obj_RF.si_eps_update,obj_RF.L_c,'mv');
    Si_Gp = lambda_t_inv*nu_t/(nu_t-2);
end

% SA mean and variance by Gauss-Legendre integration
mu_SA = 1/4*sum(mu_Gp.*w_Gp_2D(:));
V_SA_pp  = 1/16*sum(Si_Gp.*w_Gp_4D,'all');
si_SA = sqrt(V_SA_pp);

% adaption of grids
zz1_0 = zz1_SA(1:end-1,1:end-1); % z2 grid for 0 side of elements
zz1_1 = zz1_SA(2:end,2:end);   % z1 grid for 1 side of elements
zz2_0 = zz2_SA(1:end-1,1:end-1); % z2 grid for 0 side of elements
zz2_1 = zz2_SA(2:end,2:end);   % z2 grid for 1 side of elements

% SA covariance
for ii=1:size(Si_SA,1)
    
    % matrices to store variances of auxiliary domains and corresponding signs
    Delta = zeros(4,4);
    sign_Delta = zeros(4,4);
    
    if ii<id_el
        Si_SA(id_el,ii) = Si_SA(ii,id_el);
    elseif ii==id_el
        Si_SA(id_el,ii) = V_SA_pp;
    else
        
        % averaging domain of other averaging element
        z1_a_0 = zz1_0(ii);
        z1_a_1 = zz1_1(ii);
        z2_a_0 = zz2_0(ii);
        z2_a_1 = zz2_1(ii);
        
        % covariances of auxiliary domains
        for aux1=0:3
            
            % determination of auxiliary length in z1
            switch aux1
                case 0
                    z1_int_0 = min(z1_1,z1_a_0);
                    z1_int_1 = max(z1_1,z1_a_0);
                case 1
                    z1_int_0 = min(z1_0,z1_a_0);
                    z1_int_1 = max(z1_0,z1_a_0);
                case 2
                    z1_int_0 = min(z1_0,z1_a_1);
                    z1_int_1 = max(z1_0,z1_a_1);
                case 3
                    z1_int_0 = min(z1_1,z1_a_1);
                    z1_int_1 = max(z1_1,z1_a_1);
            end
            
            L1_a = z1_int_1-z1_int_0; % length in z1
            
            if L1_a==0
                continue
            end
            
            z1_a_Gp = (xi_Gp+1)*L1_a/2+z1_int_0; % Gauss-Legendre points in z1
            
            for aux2=0:3
                
                % determination of auxiliary length in z2
                switch aux2
                    case 0
                        z2_int_0 = min(z2_1,z2_a_0);
                        z2_int_1 = max(z2_1,z2_a_0);
                    case 1
                        z2_int_0 = min(z2_0,z2_a_0);
                        z2_int_1 = max(z2_0,z2_a_0);
                    case 2
                        z2_int_0 = min(z2_0,z2_a_1);
                        z2_int_1 = max(z2_0,z2_a_1);
                    case 3
                        z2_int_0 = min(z2_1,z2_a_1);
                        z2_int_1 = max(z2_1,z2_a_1);
                end
                
                L2_a  = z2_int_1-z2_int_0; % length in z2
                
                if L2_a==0
                    continue
                end
                
                z2_a_Gp = (xi_Gp+1)*L2_a/2+z2_int_0; % Gauss-Legendre points in z2
                
                [zz_a_1,zz_a_2] = meshgrid(z1_a_Gp',z2_a_Gp);
                
                if obj_RF.RF_hom

                    % correlation of quadrature nodes
                    D_Gp = zeros(size(zz_a_1(:),1),size(zz_a_1(:),1),2);
                    D_Gp(:,:,1) = pdist2(zz_a_1(:),zz_a_1(:));
                    D_Gp(:,:,2) = pdist2(zz_a_2(:),zz_a_2(:));
                    DwGp = sqrt(sum((D_Gp./permute(obj_RF.L_c,[1 3 2])).^2,3));
                    R_Gp=obj_RF.R_fun(DwGp);
                    
                    % covariance matrix of quadrature nodes
                    Si_a_Gp = obj_RF.si_n^2*R_Gp;
                    
                else % RF update
                    [~,lambda_t_a_inv,nu_t,~]=t_postpred(obj_RF.mu0,obj_RF.k0,obj_RF.a0,obj_RF.b0,obj_RF.R_fun,obj_RF.data_update,[zz_a_1(:) zz_a_2(:)],obj_RF.si_eps_update,obj_RF.L_c,'mv');
                    Si_a_Gp = lambda_t_a_inv*nu_t/(nu_t-2); % posterior predictive covariance matrix
                end

                Delta(aux1+1,aux2+1)      = L1_a^2*L2_a^2/16*sum(Si_a_Gp.*w_Gp_4D,'all');
                sign_Delta(aux1+1,aux2+1) = (-1)^aux1*(-1)^aux2;
            end
            
        end
        
        Si_SA(id_el,ii) = (l_z1^(-2)*l_z2^(-2))/4*sum(Delta.*sign_Delta,'all');
        
    end
    
end

% END
